from datetime import datetime

import pytest

from dante.sync import Dante


def test_create_dante_on_disk(tmp_path):
    db_path = tmp_path / "test.db"
    db = Dante(db_path)
    assert db is not None
    _ = db["test"]
    db.close()
    assert db_path.exists()


def test_create_collection():
    db = Dante()
    coll = db["test"]
    assert coll is not None
    x = (
        db.conn.cursor()
        .execute("select name from sqlite_master where type = 'table'")
        .fetchone()
    )
    assert x == ("test",)


def test_insert_find_one():
    db = Dante()
    coll = db["test"]

    data = {"a": 1, "b": 2}
    coll.insert(data)
    result = coll.find_one(a=1, b=2)
    assert result == data


def test_insert_find_many():
    db = Dante()
    coll = db["test"]

    obj1 = {"a": 1, "b": 2, "c": 3}
    obj2 = {"a": 1, "e": 4, "f": 5}
    coll.insert(obj1)
    coll.insert(obj2)
    result = coll.find_many(a=1)
    assert len(result) == 2


def test_find_nested():
    db = Dante()
    coll = db["test"]

    obj = {"a": {"b": {"c": 1}}}
    coll.insert(obj)
    result = coll.find_one(a__b__c=1)
    assert obj == result


def test_iteration():
    db = Dante()
    coll = db["test"]

    coll.insert({"a": 1, "b": 2, "c": 3})

    result = [d for d in coll]
    assert len(result) == 1
    assert result[0]["a"] == 1


def test_insert_datetime():
    db = Dante()
    coll = db["test"]

    data = {"a": 1, "b": datetime.now()}
    coll.insert(data)
    result = coll.find_one(a=1)
    assert datetime.fromisoformat(result["b"]) == data["b"]


def test_find_none():
    db = Dante()
    coll = db["test"]

    result = coll.find_one(a=1)
    assert result is None


def test_update():
    db = Dante()
    coll = db["test"]

    coll.insert({"a": 1, "b": 2})
    coll.update({"a": 1, "b": 3}, a=1)
    result = coll.find_one(a=1)
    assert result["b"] == 3


def test_update_without_filter_fails():
    db = Dante()
    coll = db["test"]
    with pytest.raises(ValueError):
        coll.update({})


def test_set():
    db = Dante()
    coll = db["test"]
    coll.insert({"a": 1, "b": 2})
    coll.set({"b": 3}, a=1)
    result = coll.find_one(a=1)
    assert result["b"] == 3


def test_set_nested():
    db = Dante()
    coll = db["test"]
    coll.insert({"a": {"b": 2}})
    coll.set({"a__b": 3}, a__b=2)
    result = coll.find_one(a__b=3)
    assert result["a"]["b"] == 3


def test_set_without_fields_fails():
    db = Dante()
    coll = db["test"]
    with pytest.raises(ValueError):
        coll.set({}, a=1)


def test_set_without_filter_fails():
    db = Dante()
    coll = db["test"]
    with pytest.raises(ValueError):
        coll.set({"b": 3})


def test_delete():
    db = Dante()
    coll = db["test"]

    coll.insert({"a": 1, "b": 2})
    coll.insert({"a": 1, "b": 3})
    coll.delete(a=1)
    result = coll.find_many(a=1)
    assert result == []


def test_delete_without_filter_fails():
    db = Dante()
    coll = db["test"]
    with pytest.raises(ValueError):
        coll.delete()


def test_clear():
    db = Dante()
    coll = db["test"]

    coll.insert({"a": 1, "b": 2})
    coll.clear()
    result = coll.find_many()
    assert result == []


def test_str():
    d = Dante()
    assert str(d) == '<Dante(":memory:")>'
    c = d["test"]
    assert str(c) == '<Collection(":memory:/test")>'



from __future__ import annotations

import sqlite3
from typing import Any, Iterable

from pydantic import BaseModel

from .base import BaseCollection, BaseDante


class Dante(BaseDante):
    """
    Dante, a simple synchronous database wrapper for SQLite.

    :param db_name: Name of the database, defaults to in-memory database
    :param auto_commit: Whether to automatically commit transactions, defaults to True

    Usage:

    >>> from dante import Dante
    >>> db = Dante()
    >>> coll = db.collection("test")
    >>> coll.insert({"person": "Jane", "message": "Hello World!"})
    >>> result = coll.find_one(person="Jane")
    >>> result["message"] = "Goodbye World!"
    >>> coll.update_one(result, person="Jane")
    >>> coll.delete_one(person="Jane")
    """

    def get_connection(self) -> sqlite3.Connection:
        if not self.conn:
            self.conn = sqlite3.connect(self.db_name)
        return self.conn

    def collection(self, name: str, model: BaseModel | None = None) -> "Collection":
        conn = self.get_connection()
        conn.execute(f"CREATE TABLE IF NOT EXISTS {name} (data TEXT)")
        conn.commit()
        return Collection(name, self, model)

    def commit(self):
        if self.conn:
            self.conn.commit()

    def _maybe_commit(self):
        if self.auto_commit and self.conn:
            self.commit()

    def close(self):
        if self.conn:
            self.conn.close()
            self.conn = None


class Collection(BaseCollection):
    """
    Synchronous Dante collection.

    If the pydantic model class is specified, the data is automatically
    serialized/deserialized.

    :param name: Name of the collection
    :param db: Dante instance
    :param model: Pydantic model class (if using with Pydantic)
    """

    def insert(self, data: dict[str, Any] | BaseModel):
        cursor = self.conn.cursor()
        cursor.execute(
            f"INSERT INTO {self.name} (data) VALUES (?)", (self._to_json(data),)
        )
        self.db._maybe_commit()

    def find_many(
        _self,
        _limit: int | None = None,
        /,
        **kwargs: Any,
    ) -> list[dict | BaseModel]:
        query, values = _self._build_query(_limit, **kwargs)

        cursor = _self.conn.cursor()
        cursor.execute(f"SELECT data FROM {_self.name}{query}", values)
        rows = cursor.fetchall()

        return [_self._from_json(row[0]) for row in rows]

    def find_one(_self, **kwargs: Any) -> dict | BaseModel | None:
        results = _self.find_many(1, **kwargs)
        return results[0] if len(results) > 0 else None

    def update(_self, _data: dict[str, Any] | BaseModel, /, **kwargs: Any):
        if not kwargs:
            raise ValueError("You must provide a filter to update")

        query, values = _self._build_query(None, **kwargs)

        cursor = _self.conn.cursor()
        cursor.execute(
            f"UPDATE {_self.name} SET data = ? {query}",
            (_self._to_json(_data), *values),
        )
        _self.db._maybe_commit()

    def set(_self, _fields: dict[str, Any], **kwargs: Any):
        if not _fields:
            raise ValueError("You must provide fields to set")

        if not kwargs:
            raise ValueError("You must provide a filter to update")

        set_clause, clause_values = _self._build_set_clause(**_fields)
        query, query_values = _self._build_query(None, **kwargs)

        cursor = _self.conn.cursor()
        cursor.execute(
            f"UPDATE {_self.name} {set_clause} {query}",
            *[clause_values + query_values],
        )
        _self.db._maybe_commit()

    def delete(_self, /, **kwargs: Any):
        if not kwargs:
            raise ValueError("You must provide a filter to delete")

        query, values = _self._build_query(None, **kwargs)

        cursor = _self.conn.cursor()
        cursor.execute(f"DELETE FROM {_self.name}{query}", values)
        _self.db._maybe_commit()

    def clear(self):
        cursor = self.conn.cursor()
        cursor.execute(f"DELETE FROM {self.name}")
        self.db._maybe_commit()

    def __iter__(self) -> Iterable[dict[str, Any] | BaseModel]:
        """
        Iterate over the documents in the collection.
        """
        return iter(self.find_many())



import pytest
from pydantic import BaseModel


class MyModel(BaseModel):
    a: int
    b: str = "foo"


@pytest.mark.asyncio
async def test_get_model_collection(db):
    coll = await db[MyModel]
    assert coll.model == MyModel
    assert coll.name == "MyModel"


@pytest.mark.asyncio
async def test_insert_find_one_model(db):
    coll = await db[MyModel]

    obj = MyModel(a=1)
    await coll.insert(obj)

    result = await coll.find_one(a=1)
    assert isinstance(result, MyModel)

    assert result.a == 1
    assert result.b == "foo"


@pytest.mark.asyncio
async def test_insert_find_many_model(db):
    coll = await db[MyModel]

    obj = MyModel(a=1)
    await coll.insert(obj)

    result = await coll.find_many(a=1)
    assert len(result) == 1
    assert isinstance(result[0], MyModel)

    assert result[0].a == 1
    assert result[0].b == "foo"


@pytest.mark.asyncio
async def test_update_model(db):
    coll = await db[MyModel]

    obj = MyModel(a=1)
    await coll.insert(obj)

    obj.b = "bar"
    await coll.update(obj, a=1)

    result = await coll.find_one(a=1)
    assert isinstance(result, MyModel)

    assert result.a == 1
    assert result.b == "bar"



from __future__ import annotations

from typing import Any, AsyncGenerator

import aiosqlite
from pydantic import BaseModel

from .base import BaseCollection, BaseDante


class Dante(BaseDante):
    async def get_connection(self) -> aiosqlite.Connection:
        if not self.conn:
            self.conn = await aiosqlite.connect(self.db_name)
        return self.conn

    async def collection(
        self,
        name: str,
        model: BaseModel | None = None,
    ) -> "Collection":
        conn = await self.get_connection()
        await conn.execute(f"CREATE TABLE IF NOT EXISTS {name} (data TEXT)")
        await conn.commit()
        return Collection(name, self, model)

    async def commit(self):
        if self.conn:
            await self.conn.commit()

    async def _maybe_commit(self):
        if self.auto_commit and self.conn:
            await self.commit()

    async def close(self):
        if self.conn:
            await self.conn.close()
            self.conn = None


class Collection(BaseCollection):
    """
    Asynchronous Dante collection.

    If the pydantic model class is specified, the data is automatically
    serialized/deserialized.

    :param name: Name of the collection
    :param db: Dante instance
    :param model: Pydantic model class (if using with Pydantic)
    """

    async def insert(self, data: dict[str, Any] | BaseModel):
        conn: aiosqlite.Connection = await self.db.get_connection()
        await conn.execute(
            f"INSERT INTO {self.name} (data) VALUES (?)", (self._to_json(data),)
        )
        await self.db._maybe_commit()

    async def find_many(
        _self,
        _limit: int | None = None,
        /,
        **kwargs: Any,
    ) -> list[dict[str, Any] | BaseModel]:
        query, values = _self._build_query(_limit, **kwargs)

        conn: aiosqlite.Connection = await _self.db.get_connection()
        cursor = await conn.cursor()
        await cursor.execute(f"SELECT data FROM {_self.name}{query}", values)
        rows = await cursor.fetchall()

        return [_self._from_json(row[0]) for row in rows]

    async def find_one(_self, **kwargs: Any) -> dict | BaseModel | None:
        results = await _self.find_many(1, **kwargs)
        return results[0] if len(results) > 0 else None

    async def update(_self, _data: dict[str, Any] | BaseModel, /, **kwargs: Any):
        if not kwargs:
            raise ValueError("You must provide a filter to update")

        query, values = _self._build_query(None, **kwargs)

        conn: aiosqlite.Connection = await _self.db.get_connection()
        await conn.execute(
            f"UPDATE {_self.name} SET data = ?{query}",
            (_self._to_json(_data), *values),
        )
        await _self.db._maybe_commit()

    async def set(_self, _fields: dict[str, Any], **kwargs: Any):
        if not _fields:
            raise ValueError("You must provide fields to set")

        if not kwargs:
            raise ValueError("You must provide a filter to update")

        set_clause, clause_values = _self._build_set_clause(**_fields)
        query, query_values = _self._build_query(None, **kwargs)

        conn: aiosqlite.Connection = await _self.db.get_connection()
        await conn.execute(
            f"UPDATE {_self.name} {set_clause} {query}",
            *[clause_values + query_values],
        )
        await _self.db._maybe_commit()

    async def delete(_self, /, **kwargs: Any):
        if not kwargs:
            raise ValueError("You must provide a filter to delete")

        query, values = _self._build_query(None, **kwargs)

        conn: aiosqlite.Connection = await _self.db.get_connection()
        await conn.execute(f"DELETE FROM {_self.name}{query}", values)
        await _self.db._maybe_commit()

    async def clear(self):
        conn: aiosqlite.Connection = await self.db.get_connection()
        await conn.execute(f"DELETE FROM {self.name}")
        await self.db._maybe_commit()

    async def __aiter__(self) -> AsyncGenerator[dict | BaseModel]:
        """
        Asynchronously iterate over the documents in the collection.
        """
        results = await self.find_many()
        for r in results:
            yield r



from dante import Dante

# Create 'mydatabase.db' in current directory and open it
# (you can omit the database name to create a temporary in-memory database.)
db = Dante("mydatabase.db")

# Use 'mycollection' collection (also known as a "table")
collection = db["mycollection"]

# Insert a dictionary to the database
data = {"name": "Dante", "text": "Hello World!"}
collection.insert(data)

# Find a dictionary with the specified attribute(s)
result = collection.find_one(name="Dante")
print(result["text"])

new_data = {"name": "Virgil", "text": "Hello World!"}
collection.update(new_data, name="Dante")



from asyncio import run
from dante import AsyncDante


async def main():
    db = AsyncDante("mydatabase.db")
    collection = await db["mycollection"]

    data = {"name": "Dante", "text": "Hello World!"}
    await collection.insert(data)

    result = await collection.find_one(name="Dante")
    print(result["text"])

    new_data = {"name": "Virgil", "text": "Hello World!"}
    await collection.update(new_data, name="Dante")

    await db.close()


run(main())



from dante import Dante
from pydantic import BaseModel


class Message(BaseModel):
    name: str
    text: str


# Open the database and get the collection for messages
db = Dante("mydatabase.db")
collection = db[Message]

# Insert a model to the database
obj = Message(name="Dante", text="Hello world!")
collection.insert(obj)

# Find a model with the specified attribute(s)
result = collection.find_one(name="Dante")
print(result.text)

# Find a model in the collection with the attribute name=Dante
# and update (overwrite) it with the new model data
result.name = "Virgil"
collection.update(result, name="Dante")



import sys

from dante import Dante

db = Dante("todo.db")
collection = db["todos"]


def help():
    print("Usage: todo.py <command> [args...]")
    print("Commands:")
    print("  list")
    print("  add <todo>")
    print("  remove <todo>")
    print("  done <todo>")
    print("  undone <todo>")


def main():
    if len(sys.argv) < 2:
        help()
        sys.exit(-1)

    command = sys.argv[1]

    if command == "list":
        for i, todo in enumerate(collection):
            print(f"{i+1}. {todo['text']}{' (done)' if todo['done'] else ''}")

        return
    elif command == "help":
        help()
        return

    if len(sys.argv) < 3:
        print("Missing argument: todo")
        sys.exit(-1)

    todo = sys.argv[2]

    if command == "add":
        collection.insert({"text": todo, "done": False})
    elif command == "remove":
        collection.delete(text=todo)
    elif command == "done":
        collection.set({"done": True}, text=todo)
    elif command == "undone":
        collection.set({"done": False}, text=todo)
    else:
        print(f"Unknown command: {command} (try 'help' to see available commands)")
        sys.exit(-1)


main()



from __future__ import annotations

import json
from abc import ABC, abstractmethod
from datetime import date, datetime
from typing import Any

from pydantic import BaseModel


class DanteEncoder(json.JSONEncoder):
    """
    Custom JSON encoder for handling datetime and date objects.

    This encoder extends the default JSON encoder to properly serialize
    datetime and date objects by converting them to ISO format strings.
    """

    def default(self, obj: Any) -> str | Any:
        """
        Encode datetime and date objects as ISO format strings.

        :param obj: The object to be encoded.

        :returns: JSON-serializable representation of obj
        """
        if isinstance(obj, datetime) or isinstance(obj, date):
            return obj.isoformat()
        return super().default(obj)  # pragma: no cover


class BaseDante(ABC):
    """
    Base class for Dante database operations.

    :param db_name: Name of the database, defaults to in-memory database
    :param auto_commit: Whether to automatically commit transactions, defaults to True
    """

    MEMORY = ":memory:"

    def __init__(self, db_name: str = MEMORY, auto_commit: bool = True):
        """
        Initialize the Dante instance.

        :param db_name: Name of the database, defaults to in-memory database
        :param auto_commit: Whether to automatically commit transactions, defaults to True
        """
        self.db_name = db_name
        self.conn = None
        self.auto_commit = auto_commit

    def __str__(self) -> str:
        """
        Return a string representation of the Dante instance.

        :return: String representation of the instance
        """
        return f'<{self.__class__.__name__}("{self.db_name}")>'

    @abstractmethod
    def get_connection(self):
        """
        Get a connection to the database.

        :return: Database connection object
        """

    @abstractmethod
    def collection(self, name: str, model: type | None = None) -> "BaseCollection":
        """
        Get a collection from the database.

        :param name: Name of the collection
        :param model: Pydantic model class (if using with Pydantic)

        :return: Collection instance
        """

    def __getitem__(self, name: str | type) -> "BaseCollection":
        """
        Get a collection using dictionary-like syntax.

        :param name: Name of the collection or a Pydantic model class
        :return: Collection instance
        """
        if isinstance(name, type) and issubclass(name, BaseModel):
            return self.collection(name.__name__, name)
        else:
            return self.collection(name)

    @abstractmethod
    def commit(self):
        """
        Commit the current transaction (if auto-commit is disabled).

        This method is a no-op if auto-commit is enabled.
        """

    @abstractmethod
    def close(self):
        """
        Close the database connection.

        This method should be called when the database is no longer needed.
        """


class BaseCollection(ABC):
    """
    Base class for collection operations in the Dante database.

    This class provides methods for interacting with collections in the database,
    including serialization and deserialization of data, and query building for
    find, update and delete methods.

    :param name: Name of the collection
    :param db: Dante instance representing the database
    :param model: Optional Pydantic model class for data validation and serialization
    """

    def __init__(self, name: str, db: BaseDante, model: BaseModel | None = None):
        """
        Initialize the BaseCollection instance.

        :param name: Name of the collection
        :param db: BaseDante instance representing the database
        :param model: Optional Pydantic model class for data validation and serialization
        """
        self.name = name
        self.db = db
        self.conn = db.conn
        self.model = model

    def __str__(self) -> str:
        """
        Return a string representation of the BaseCollection instance.

        :return: String representation of the instance
        """
        return f'<{self.__class__.__name__}("{self.db.db_name}/{self.name}")>'

    def _to_json(self, data: dict | BaseModel) -> str:
        """
        Internal method to serialize data to JSON before saving.

        :param data: Data or Pydantic object to serialize
        :return: JSON-serialized data
        """
        if self.model and isinstance(data, BaseModel):
            return data.model_dump_json()
        else:
            return json.dumps(data, cls=DanteEncoder)

    def _from_json(self, json_text: str) -> dict | BaseModel:
        """
        Internal method to parse JSON data after loading.

        :param json_text: Raw JSON data to parse
        :return: Deserialized data as dictionary or Pydantic model
        """
        data = json.loads(json_text)
        return self.model(**data) if self.model else data

    def _build_query(_self, _limit: int | None, /, **kwargs: Any) -> tuple[str, list]:
        """
        Internal method to create an SQL WHERE/LIMIT clauses.

        Builds the query from key/value pairs with optional limit.

        :param _limit: Optional LIMIT clause
        :param kwargs: key/value pairs to search for
        :return: fragments of query to prepare, with corresponding values

        """
        values = []
        if kwargs:
            query_parts = []
            for key, value in kwargs.items():
                key = "$." + key.replace("__", ".")
                query_parts.append("data->>? = ?")
                values.extend([key, value])
            query = " WHERE " + " AND ".join(query_parts)
        else:
            query = ""

        if _limit:
            query += " LIMIT ?"
            values.append(_limit)

        return query, values

    def _build_set_clause(_self, **kwargs: Any) -> tuple[str, list]:
        """
        Internal method to create an SQL SET clause.

        Builds the SET clause from key/value pairs.

        :param kwargs: key/value pairs to set
        :return: fragments of query to prepare, with corresponding values
        """
        clause_parts = []
        values = []
        for key, value in kwargs.items():
            key = key.replace("__", ".")
            clause_parts.append("?, ?")
            values.extend(["$." + key, value])
        clause = "SET data = json_set(data, " + ", ".join(clause_parts) + ")"

        return clause, values

    @abstractmethod
    def insert(self, data: dict | BaseModel):
        """
        Insert data into the collection.

        :param data: Data to insert
        """

    @abstractmethod
    def find_many(
        _self,
        _limit: int | None = None,
        /,
        **kwargs: Any,
    ) -> list[dict | BaseModel]:
        """
        Find documents matching the query.

        :param _limit: Maximum number of documents to return
        :param kwargs: Fields to match in the documents
        :return: List of documents matching the query
        """

    @abstractmethod
    def find_one(_self, **kwargs: Any) -> dict | BaseModel | None:
        """
        Find a single document matching the query.

        If multiple documents match the query, an arbitrary matching
        document is returned.

        :param kwargs: Fields to match in the document
        :return: Document matching the query, or None if not found
        """

    @abstractmethod
    def update(_self, _data: dict | BaseModel, /, **kwargs: Any):
        """
        Update documents matching the query.

        Note that the data must be a full object, not just the fields to update.

        :param _data: Data to update with (must be a full object)
        :param kwargs: Fields to match in the documents
        """

    @abstractmethod
    def delete(_self, /, **kwargs: Any):
        """
        Delete documents matching the query.

        :param kwargs: Fields to match in the documents
        """

    @abstractmethod
    def set(_self, _fields: dict[str, Any], **kwargs: Any):
        """
        Update specific fields in documents matching the query.

        :param _fields: Fields to update
        :param kwargs: Fields to match in the documents
        """

    @abstractmethod
    def clear(self):
        """
        Delete all documents in the collection.
        """



from asyncio import gather
from datetime import datetime

import pytest

from dante import AsyncDante


@pytest.mark.asyncio
async def test_create_dante_on_disk(tmp_path):
    db_path = tmp_path / "test.db"
    db = AsyncDante(db_path)
    assert db is not None

    assert not db_path.exists()

    coll = await db["test"]
    assert coll is not None
    assert db_path.exists()
    await db.close()


@pytest.mark.asyncio
async def test_create_collection(db):
    coll = await db["test"]
    assert coll is not None
    q = await db.conn.execute("select name from sqlite_master where type = 'table'")
    x = await q.fetchone()
    assert x == ("test",)


@pytest.mark.asyncio
async def test_insert_find_one(db):
    coll = await db["test"]

    data = {"a": 1, "b": 2}
    await coll.insert(data)
    result = await coll.find_one(a=1, b=2)
    assert result == data


@pytest.mark.asyncio
async def test_insert_find_many(db):
    coll = await db["test"]

    obj1 = {"a": 1, "b": 2, "c": 3}
    obj2 = {"a": 1, "e": 4, "f": 5}
    await gather(
        coll.insert(obj1),
        coll.insert(obj2),
    )
    result = await coll.find_many(a=1)
    assert len(result) == 2


@pytest.mark.asyncio
async def test_insert_datetime(db):
    coll = await db["test"]

    data = {"a": 1, "b": datetime.now()}
    await coll.insert(data)
    result = await coll.find_one(a=1)
    assert datetime.fromisoformat(result["b"]) == data["b"]


@pytest.mark.asyncio
async def test_find_none(db):
    coll = await db["test"]

    result = await coll.find_one(a=1)
    assert result is None


@pytest.mark.asyncio
async def test_iteration(db):
    coll = await db["test"]

    await coll.insert({"a": 1})

    count = 0
    async for data in coll:
        count += 1
        assert data["a"] == 1

    assert count == 1


@pytest.mark.asyncio
async def test_update(db):
    coll = await db["test"]

    await coll.insert({"a": 1, "b": 2})
    await coll.update({"a": 1, "b": 3}, a=1)
    result = await coll.find_one(a=1)
    assert result["b"] == 3


@pytest.mark.asyncio
async def test_update_without_filter_fails(db):
    coll = await db["test"]

    await coll.insert({"a": 1, "b": 2})
    with pytest.raises(ValueError):
        await coll.update({})


@pytest.mark.asyncio
async def test_set(db):
    coll = await db["test"]
    await coll.insert({"a": 1, "b": 2})
    await coll.set({"b": 3}, a=1)
    result = await coll.find_one(a=1)
    assert result["b"] == 3


@pytest.mark.asyncio
async def test_set_without_fields_fails(db):
    coll = await db["test"]
    with pytest.raises(ValueError):
        await coll.set({}, a=1)


@pytest.mark.asyncio
async def test_set_without_filter_fails(db):
    coll = await db["test"]
    with pytest.raises(ValueError):
        await coll.set({"a": 1})


@pytest.mark.asyncio
async def test_delete(db):
    coll = await db["test"]

    await gather(
        coll.insert({"a": 1, "b": 2}),
        coll.insert({"a": 1, "b": 3}),
    )
    await coll.delete(a=1)
    result = await coll.find_many(a=1)
    assert result == []


@pytest.mark.asyncio
async def test_delete_without_filter_fails(db):
    coll = await db["test"]

    with pytest.raises(ValueError):
        await coll.delete()


@pytest.mark.asyncio
async def test_clear(db):
    coll = await db["test"]

    await coll.insert({"a": 1, "b": 2})
    await coll.clear()
    result = await coll.find_many()
    assert result == []



import pytest
from pydantic import BaseModel

from dante import Dante


class MyModel(BaseModel):
    a: int
    b: str = "foo"


def test_get_model_collection():
    db = Dante()
    coll = db[MyModel]
    assert coll.model == MyModel
    assert coll.name == "MyModel"


def test_insert_find_one_model():
    db = Dante()
    coll = db[MyModel]

    obj = MyModel(a=1)
    coll.insert(obj)

    result = coll.find_one(a=1)
    assert isinstance(result, MyModel)

    assert result.a == 1
    assert result.b == "foo"


def test_insert_find_many_model():
    db = Dante()
    coll = db[MyModel]

    obj = MyModel(a=1)
    coll.insert(obj)

    result = coll.find_many(a=1)
    assert len(result) == 1
    assert isinstance(result[0], MyModel)

    assert result[0].a == 1
    assert result[0].b == "foo"


def test_update_model():
    db = Dante()
    coll = db[MyModel]

    obj = MyModel(a=1)
    coll.insert(obj)

    obj.b = "bar"
    coll.update(obj, a=1)

    result = coll.find_one(a=1)
    assert isinstance(result, MyModel)

    assert result.a == 1
    assert result.b == "bar"


def test_update_without_filter_fails():
    db = Dante()
    coll = db[MyModel]

    obj = MyModel(a=1)

    with pytest.raises(ValueError):
        coll.update(obj)


def test_delete_without_filter_fails():
    db = Dante()
    coll = db[MyModel]

    with pytest.raises(ValueError):
        coll.delete()
